/////////This class represents all Tiles//////////

import waba.fx.*;
import waba.util.*;
import waba.sys.*;

//making classes and methods final speeds up things
public final class Grid  
{
	private int MAX = Memo24.MAX;		
	private static final int FIRST = 1;
	private static final int SECOND = 2;
	
	private int clickState, clickedTileIndex;	
	private int firstTileIndex, secondTileIndex;
	private int clicks, correct;
	private boolean coverUp;
	
	private Tile tile[] = new Tile[MAX];
	private int[] imageIndices = new int[MAX];
	
	public Grid()
	{				
		getNewNumbers();	
		
		for(int i=0; i<MAX; i++)				
			tile[i] = new Tile(i, imageIndices[i]);		
	}
	
	public final void cover(int tileIndex)
	{									
		tile[tileIndex].setOpen(false);		
	}
	
	private final void getNewNumbers()
    {    	
    	int i, j;
    	Random random = new Random();
    	
        //fill our array with random numbers, guarantee different numbers     
        for (i=0; i<MAX; i++)
        {
            //Random number from 0 to MAX-1            
            imageIndices[i] = random.getRandom(MAX);

            if (i>0)            
                for (j=i; j>0 ;j--)                
                    if (imageIndices[i] == imageIndices[j-1])
                        i--;              
         }
    }
		
	public final int clickedBoard(int x, int y)
	{		
		for(int i=0; i<MAX; i++)		
			if(tile[i].clickedAt(x, y))		
			{	
				clickedTileIndex = tile[i].getIndex();			
				break;
			}
				
		//don't click tiles that are alredy correctly uncovered
		if(!tile[clickedTileIndex].isCorrect())
		{	
			//ignore second click at the first tile			
			if(clickState == FIRST && firstTileIndex != clickedTileIndex)
			{
				clickState = SECOND;
				clicks ++;
			}
			else				
				clickState = FIRST;			
				
			if(clickState == FIRST)
			{
				firstTileIndex = clickedTileIndex;
				tile[clickedTileIndex].setOpen(true);
				
				coverTiles(clickedTileIndex);		
				coverUp = false;						
			}
				
			if(clickState == SECOND)
			{
				secondTileIndex = clickedTileIndex;				
	
				//images match, set them to correct
				if(tile[firstTileIndex].getImageIndex() == tile[secondTileIndex].getImageIndex())
				{
					tile[firstTileIndex].setCorrect();
					tile[secondTileIndex].setCorrect();
					correct ++;
					coverUp = false;
				}	
				else 
					coverUp = true;
				
				tile[clickedTileIndex].setOpen(true);												
			}
		}			
		return clickedTileIndex;		
	}  
	
	public final boolean triggerCoverUp()
	{		
		return coverUp;	
	}
		
	public final void coverTiles(int justClicked)
	{
		//cover all tiles except for the one just clicked and correctly uncovered ones
		for(int i=0; i<MAX; i++)		
			if(i != justClicked && !tile[i].isCorrect())				
				tile[i].setOpen(false);
				
		if(justClicked == -1)
			coverUp = false;	
	}	
	
	public final Tile getTile(int index)
	{			
		return tile[index];
	}
	
	public final int getClicks()
	{
		return clicks;
	}
	
	public final boolean allCorrect()
	{
		if(correct == 12)
			return true;
			
		return false;
	}
}
